@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2019  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -68,7 +70,7 @@ class AttachmentsController < ApplicationController
       @attachment.increment_download
     end
 
-    if stale?(:etag => @attachment.digest)
+    if stale?(:etag => @attachment.digest, :template => false)
       # images are sent inline
       send_file @attachment.diskfile, :filename => filename_for_content_disposition(@attachment.filename),
                                       :type => detect_content_type(@attachment),
@@ -78,11 +80,12 @@ class AttachmentsController < ApplicationController
 
   def thumbnail
     if @attachment.thumbnailable? && tbnail = @attachment.thumbnail(:size => params[:size])
-      if stale?(:etag => tbnail)
-        send_file tbnail,
+      if stale?(:etag => tbnail, :template => false)
+        send_file(
+          tbnail,
           :filename => filename_for_content_disposition(@attachment.filename),
-          :type => detect_content_type(@attachment),
-          :disposition => 'inline'
+          :type => detect_content_type(@attachment, true),
+          :disposition => 'inline')
       end
     else
       # No thumbnail for the attachment or thumbnail could not be created
@@ -164,8 +167,10 @@ class AttachmentsController < ApplicationController
 
   # Returns the menu item that should be selected when viewing an attachment
   def current_menu_item
-    if @attachment
-      case @attachment.container
+    container = @attachment.try(:container) || @container
+
+    if container
+      case container
       when WikiPage
         :wiki
       when Message
@@ -173,7 +178,7 @@ class AttachmentsController < ApplicationController
       when Project, Version
         :files
       else
-        @attachment.container.class.name.pluralize.downcase.to_sym
+        container.class.name.pluralize.downcase.to_sym
       end
     end
   end
@@ -232,12 +237,20 @@ class AttachmentsController < ApplicationController
     @attachment.deletable? ? true : deny_access
   end
 
-  def detect_content_type(attachment)
+  def detect_content_type(attachment, is_thumb = false)
     content_type = attachment.content_type
     if content_type.blank? || content_type == "application/octet-stream"
-      content_type = Redmine::MimeType.of(attachment.filename)
+      content_type =
+        Redmine::MimeType.of(attachment.filename).presence ||
+        "application/octet-stream"
     end
-    content_type.presence || "application/octet-stream"
+
+    if is_thumb && content_type == "application/pdf"
+      # PDF previews are stored in PNG format
+      content_type = "image/png"
+    end
+
+    content_type
   end
 
   def disposition(attachment)
