@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2019  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -67,6 +69,26 @@ class QueriesControllerTest < Redmine::ControllerTest
     assert_response 404
   end
 
+  def test_new_should_not_render_show_inline_columns_option_for_query_without_available_inline_columns
+    @request.session[:user_id] = 1
+    get :new, :params => {
+        :type => 'ProjectQuery'
+      }
+
+    assert_response :success
+    assert_select 'p[class=?]', 'block_columns', 0
+  end
+
+  def test_new_should_not_render_show_totals_option_for_query_without_totable_columns
+    @request.session[:user_id] = 1
+    get :new, :params => {
+        :type => 'ProjectQuery'
+      }
+
+    assert_response :success
+    assert_select 'p[class=?]', 'totables_columns', 0
+  end
+
   def test_new_time_entry_query
     @request.session[:user_id] = 2
     get :new, :params => {
@@ -75,6 +97,39 @@ class QueriesControllerTest < Redmine::ControllerTest
       }
     assert_response :success
     assert_select 'input[name=type][value=?]', 'TimeEntryQuery'
+    assert_select 'p[class=?]', 'totable_columns', 1
+    assert_select 'p[class=?]', 'block_columns', 0
+  end
+
+  def test_new_project_query_for_projects
+    @request.session[:user_id] = 1
+    get :new, :params => {
+        :type => 'ProjectQuery'
+      }
+    assert_response :success
+    assert_select 'input[name=type][value=?]', 'ProjectQuery'
+  end
+
+  def test_new_project_query_should_not_render_roles_visibility_options
+    @request.session[:user_id] = 1
+    get :new, :params => {
+        :type => 'ProjectQuery'
+      }
+
+    assert_response :success
+    assert_select 'input[id=?]', 'query_visibility_0', 1
+    assert_select 'input[id=?]', 'query_visibility_2', 1
+    assert_select 'input[id=?]', 'query_visibility_1', 0
+  end
+
+  def test_new_project_query_should_not_render_for_all_projects_option
+    @request.session[:user_id] = 1
+    get :new, :params => {
+        :type => 'ProjectQuery'
+      }
+
+    assert_response :success
+    assert_select 'input[name=?]', 'for_all_projects', 0
   end
 
   def test_new_time_entry_query_should_select_spent_time_from_main_menu
@@ -293,7 +348,8 @@ class QueriesControllerTest < Redmine::ControllerTest
           :query => {
             :name => "test_create_from_gantt",
             :draw_relations => '1',
-            :draw_progress_line => '1'
+            :draw_progress_line => '1',
+            :draw_selected_columns => '1'
           }
         }
       assert_response 302
@@ -302,6 +358,7 @@ class QueriesControllerTest < Redmine::ControllerTest
     assert_redirected_to "/issues/gantt?query_id=#{query.id}"
     assert_equal true, query.draw_relations
     assert_equal true, query.draw_progress_line
+    assert_equal true, query.draw_selected_columns
   end
 
   def test_create_project_query_from_gantt
@@ -319,7 +376,8 @@ class QueriesControllerTest < Redmine::ControllerTest
           :query => {
             :name => "test_create_from_gantt",
             :draw_relations => '0',
-            :draw_progress_line => '0'
+            :draw_progress_line => '0',
+            :draw_selected_columns => '0'
           }
         }
       assert_response 302
@@ -328,6 +386,7 @@ class QueriesControllerTest < Redmine::ControllerTest
     assert_redirected_to "/projects/ecookbook/issues/gantt?query_id=#{query.id}"
     assert_equal false, query.draw_relations
     assert_equal false, query.draw_progress_line
+    assert_equal false, query.draw_selected_columns
   end
 
   def test_create_project_public_query_should_force_private_without_manage_public_queries_permission
@@ -435,6 +494,32 @@ class QueriesControllerTest < Redmine::ControllerTest
     assert q.valid?
   end
 
+  def test_create_public_project_query
+    @request.session[:user_id] = 1
+
+    q = new_record(ProjectQuery) do
+      post :create, :params => {
+          :type => 'ProjectQuery',
+          :default_columns => '1',
+          :f => ["status"],
+          :op => {
+            "status" => "="
+          },
+          :v => {
+            "status" => ['1']
+          },
+          :query => {
+            "name" => "test_new_project_public_query", "visibility" => "2"
+          }
+        }
+    end
+
+    assert_redirected_to :controller => 'projects', :action => 'index', :query_id => q.id
+
+    assert q.is_public?
+    assert q.valid?
+  end
+
   def test_edit_global_public_query
     @request.session[:user_id] = 1
     get :edit, :params => {
@@ -671,11 +756,12 @@ class QueriesControllerTest < Redmine::ControllerTest
     assert_equal 'application/json', response.content_type
     json = ActiveSupport::JSON.decode(response.body)
 
-    assert_equal 6, json.count
+    assert_equal 7, json.count
     # "me" value should not be grouped
     assert_include ["<< me >>", "me"], json
     assert_include ["Dave Lopper", "3", "active"], json
     assert_include ["Dave2 Lopper2", "5", "locked"], json
+    assert_include ["Anonymous", User.anonymous.id.to_s], json
   end
 
   def test_user_filter_should_return_active_and_locked_users_grouped_by_status
@@ -689,7 +775,7 @@ class QueriesControllerTest < Redmine::ControllerTest
     assert_equal 'application/json', response.content_type
     json = ActiveSupport::JSON.decode(response.body)
 
-    assert_equal 6, json.count
+    assert_equal 7, json.count
     # "me" value should not be grouped
     assert_include ["<< me >>", "me"], json
     assert_include ["Dave Lopper", "3", "active"], json
