@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2019  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,12 +26,9 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
            :repositories, :enabled_modules
 
   REPOSITORY_PATH = Rails.root.join('tmp/test/mercurial_repository').to_s
-  CHAR_1_HEX = "\xc3\x9c"
   PRJ_ID     = 3
   NUM_REV    = 34
 
-  ruby19_non_utf8_pass = Encoding.default_external.to_s != 'UTF-8'
-
   def setup
     super
     User.current = nil
@@ -41,13 +40,9 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
                       )
     assert @repository
     @diff_c_support = true
-    @char_1        = CHAR_1_HEX.dup.force_encoding('UTF-8')
-    @tag_char_1    = "tag-#{CHAR_1_HEX}-00".force_encoding('UTF-8')
-    @branch_char_0 = "branch-#{CHAR_1_HEX}-00".force_encoding('UTF-8')
-    @branch_char_1 = "branch-#{CHAR_1_HEX}-01".force_encoding('UTF-8')
   end
 
-  if ruby19_non_utf8_pass
+  if Encoding.default_external.to_s != 'UTF-8'
     puts "TODO: Mercurial functional test fails " +
          "when Encoding.default_external is not UTF-8. " +
          "Current value is '#{Encoding.default_external.to_s}'"
@@ -184,9 +179,9 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
         assert_select 'table.entries tbody' do
           assert_select 'tr', 4
           assert_select 'tr.file td.filename a', :text => "make-latin-1-file.rb"
-          assert_select 'tr.file td.filename a', :text => "test-#{@char_1}-1.txt"
-          assert_select 'tr.file td.filename a', :text => "test-#{@char_1}-2.txt"
-          assert_select 'tr.file td.filename a', :text => "test-#{@char_1}.txt"
+          assert_select 'tr.file td.filename a', :text => "test-Ü-1.txt"
+          assert_select 'tr.file td.filename a', :text => "test-Ü-2.txt"
+          assert_select 'tr.file td.filename a', :text => "test-Ü.txt"
         end
 
         assert_select 'table.changesets tbody' do
@@ -217,11 +212,11 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-       [
+      [
           'default',
-          @branch_char_1,
+          'branch-Ü-01',
           'branch (1)[2]&,%.-3_4',
-          @branch_char_0,
+          'branch-Ü-00',
           'test_branch.latin-1',
           'test-branch-00',
       ].each do |bra|
@@ -242,8 +237,8 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-       [
-        @tag_char_1,
+      [
+        'tag-Ü-00',
         'tag_test.00',
         'tag-init-revision'
       ].each do |tag|
@@ -285,7 +280,7 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
         get :entry, :params => {
             :id => PRJ_ID,
             :repository_id => @repository.id,
-          :path => repository_path_hash(['latin-1-dir', "test-#{@char_1}-2.txt"])[:param],
+          :path => repository_path_hash(['latin-1-dir', "test-Ü-2.txt"])[:param],
           :rev => r1
           }
         assert_response :success
@@ -299,11 +294,11 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
           get :entry, :params => {
               :id => PRJ_ID,
               :repository_id => @repository.id,
-            :path => repository_path_hash(['latin-1-dir', "test-#{@char_1}.txt"])[:param],
+            :path => repository_path_hash(['latin-1-dir', "test-Ü.txt"])[:param],
             :rev => r1
             }
           assert_response :success
-          assert_select 'tr#L1 td.line-code', :text => /test-#{@char_1}.txt/
+          assert_select 'tr#L1 td.line-code', :text => /test-Ü.txt/
         end
       end
     end
@@ -393,7 +388,7 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
               }
             assert_response :success
             assert_select 'table' do
-              assert_select 'thead th.filename', :text => /latin-1-dir\/test-#{@char_1}-2.txt/
+              assert_select 'thead th.filename', :text => /latin-1-dir\/test-Ü-2.txt/
               assert_select 'tbody td.diff_in', :text => /It is written in Python/
             end
           end
@@ -476,7 +471,7 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
         get :annotate, :params => {
             :id => PRJ_ID,
             :repository_id => @repository.id,
-          :path => repository_path_hash(['latin-1-dir', "test-#{@char_1}-2.txt"])[:param],
+          :path => repository_path_hash(['latin-1-dir', "test-Ü-2.txt"])[:param],
           :rev => r1
           }
         assert_response :success
@@ -498,10 +493,10 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
           get :annotate, :params => {
               :id => PRJ_ID,
               :repository_id => @repository.id,
-            :path => repository_path_hash(['latin-1-dir', "test-#{@char_1}.txt"])[:param],
+            :path => repository_path_hash(['latin-1-dir', "test-Ü.txt"])[:param],
             :rev => r1
             }
-          assert_select 'tr#L1 td.line-code', :text => /test-#{@char_1}.txt/
+          assert_select 'tr#L1 td.line-code', :text => /test-Ü.txt/
         end
       end
     end
@@ -521,7 +516,7 @@ class RepositoriesMercurialControllerTest < Redmine::RepositoryControllerTest
           assert_response :success
           assert_select 'title',
                         :text => 'Revision 1:9d5b5b004199 - Added 2 files and modified one. - eCookbook Subproject 1 - Redmine'
-          end
+        end
       end
     end
 
