@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2019  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -36,7 +38,7 @@ class ProjectCopyTest < ActiveSupport::TestCase
            :boards, :messages,
            :repositories,
            :news, :comments,
-           :documents
+           :documents, :attachments
 
   def setup
     User.current = nil
@@ -53,6 +55,7 @@ class ProjectCopyTest < ActiveSupport::TestCase
   end
 
   test "#copy should copy project attachments" do
+    set_tmp_attachments_directory
     Attachment.create!(:container => @source_project, :file => uploaded_test_file("testfile.txt", "text/plain"), :author_id => 1)
     assert @project.copy(@source_project)
 
@@ -198,13 +201,14 @@ class ProjectCopyTest < ActiveSupport::TestCase
 
     # Second issue with a cross project relation
     assert_equal 2, copied_second_issue.relations.size, "Relation not copied"
-    copied_relation = copied_second_issue.relations.select {|r| r.relation_type == 'duplicates'}.first
+    copied_relation = copied_second_issue.relations.find {|r| r.relation_type == 'duplicates'}
     assert_equal "duplicates", copied_relation.relation_type
     assert_equal 1, copied_relation.issue_from_id, "Cross project relation not kept"
     assert_not_equal source_relation_cross_project.id, copied_relation.id
   end
 
   test "#copy should copy issue attachments" do
+    set_tmp_attachments_directory
     issue = Issue.generate!(:subject => "copy with attachment", :tracker_id => 1, :project_id => @source_project.id)
     Attachment.create!(:container => issue, :file => uploaded_test_file("testfile.txt", "text/plain"), :author_id => 1)
     @source_project.issues << issue
@@ -294,6 +298,7 @@ class ProjectCopyTest < ActiveSupport::TestCase
   end
 
   test "#copy should copy version attachments" do
+    set_tmp_attachments_directory
     version = Version.generate!(:name => "copy with attachment")
     Attachment.create!(:container => version, :file => uploaded_test_file("testfile.txt", "text/plain"), :author_id => 1)
     @source_project.versions << version
@@ -369,13 +374,14 @@ class ProjectCopyTest < ActiveSupport::TestCase
     source_project = Project.find(1)
     assert @project.copy(source_project)
 
-    assert_equal 2, @project.documents.size
+    assert_equal 3, @project.documents.size
     @project.documents.each do |document|
       assert !source_project.documents.include?(document)
     end
   end
 
   test "#copy should copy document attachments" do
+    set_tmp_attachments_directory
     document = Document.generate!(:title => "copy with attachment", :category_id => 1, :project_id => @source_project.id)
     Attachment.create!(:container => document, :file => uploaded_test_file("testfile.txt", "text/plain"), :author_id => 1)
     @source_project.documents << document
